@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,25 +19,52 @@
 
 """Exceptions related to config parsing."""
 
+import typing
 import attr
 
-from qutebrowser.utils import jinja
+from qutebrowser.utils import jinja, usertypes, log
 
 
 class Error(Exception):
 
     """Base exception for config-related errors."""
 
-    pass
+
+class NoAutoconfigError(Error):
+
+    """Raised when this option can't be set in autoconfig.yml."""
+
+    def __init__(self, name: str) -> None:
+        super().__init__("The {} setting can only be set in config.py!"
+                         .format(name))
 
 
 class BackendError(Error):
 
     """Raised when this setting is unavailable with the current backend."""
 
-    def __init__(self, name, backend):
-        super().__init__("The {} setting is not available with the {} "
-                         "backend!".format(name, backend.name))
+    def __init__(
+            self, name: str,
+            backend: usertypes.Backend,
+            raw_backends: typing.Optional[typing.Mapping[str, bool]]
+    ) -> None:
+        if raw_backends is None or not raw_backends[backend.name]:
+            msg = ("The {} setting is not available with the {} backend!"
+                   .format(name, backend.name))
+        else:
+            msg = ("The {} setting needs {} with the {} backend!"
+                   .format(name, raw_backends[backend.name], backend.name))
+
+        super().__init__(msg)
+
+
+class NoPatternError(Error):
+
+    """Raised when the given setting does not support URL patterns."""
+
+    def __init__(self, name: str) -> None:
+        super().__init__("The {} setting does not support URL patterns!"
+                         .format(name))
 
 
 class ValidationError(Error):
@@ -49,7 +76,8 @@ class ValidationError(Error):
         msg: Additional error message.
     """
 
-    def __init__(self, value, msg):
+    def __init__(self, value: typing.Any,
+                 msg: typing.Union[str, Exception]) -> None:
         super().__init__("Invalid value '{}' - {}".format(value, msg))
         self.option = None
 
@@ -63,7 +91,9 @@ class NoOptionError(Error):
 
     """Raised when an option was not found."""
 
-    def __init__(self, option, *, deleted=False, renamed=None):
+    def __init__(self, option: str, *,
+                 deleted: bool = False,
+                 renamed: str = None) -> None:
         if deleted:
             assert renamed is None
             suffix = ' (this option was removed from qutebrowser)'
@@ -87,14 +117,18 @@ class ConfigErrorDesc:
         traceback: The formatted traceback of the exception.
     """
 
-    text = attr.ib()
-    exception = attr.ib()
-    traceback = attr.ib(None)
+    text = attr.ib()  # type: str
+    exception = attr.ib()  # type: typing.Union[str, Exception]
+    traceback = attr.ib(None)  # type: str
 
-    def __str__(self):
+    def __str__(self) -> str:
+        if self.traceback:
+            return '{} - {}: {}'.format(self.text,
+                                        self.exception.__class__.__name__,
+                                        self.exception)
         return '{}: {}'.format(self.text, self.exception)
 
-    def with_text(self, text):
+    def with_text(self, text: str) -> 'ConfigErrorDesc':
         """Get a new ConfigErrorDesc with the given text appended."""
         return self.__class__(text='{} ({})'.format(self.text, text),
                               exception=self.exception,
@@ -105,13 +139,19 @@ class ConfigFileErrors(Error):
 
     """Raised when multiple errors occurred inside the config."""
 
-    def __init__(self, basename, errors):
+    def __init__(self,
+                 basename: str,
+                 errors: typing.Sequence[ConfigErrorDesc]) -> None:
         super().__init__("Errors occurred while reading {}:\n{}".format(
             basename, '\n'.join('  {}'.format(e) for e in errors)))
         self.basename = basename
         self.errors = errors
+        for err in errors:
+            if err.traceback:
+                log.config.debug("Config error stack:")
+                log.config.debug(err.traceback)
 
-    def to_html(self):
+    def to_html(self) -> str:
         """Get the error texts as a HTML snippet."""
         template = jinja.environment.from_string("""
         Errors occurred while reading {{ basename }}:
