@@ -145,6 +145,18 @@ class MainWindow(QWidget):
         _private: Whether the window is in private browsing mode.
     """
 
+    # Application wide stylesheets
+    STYLESHEET = """
+        HintLabel {
+            background-color: {{ conf.colors.hints.bg }};
+            color: {{ conf.colors.hints.fg }};
+            font: {{ conf.fonts.hints }};
+            border: {{ conf.hints.border }};
+            padding-left: 3px;
+            padding-right: 3px;
+        }
+    """
+
     def __init__(self, *, private, geometry=None, parent=None):
         """Create a new main window.
 
@@ -240,6 +252,7 @@ class MainWindow(QWidget):
         self._set_decoration(config.val.window.hide_decoration)
 
         self.state_before_fullscreen = self.windowState()
+        config.set_register_stylesheet(self)
 
     def _init_geometry(self, geometry):
         """Initialize the window geometry or load it from disk."""
@@ -474,8 +487,7 @@ class MainWindow(QWidget):
         # statusbar
         self.tabbed_browser.current_tab_changed.connect(status.on_tab_changed)
 
-        self.tabbed_browser.cur_progress.connect(status.prog.setValue)
-        self.tabbed_browser.cur_load_finished.connect(status.prog.hide)
+        self.tabbed_browser.cur_progress.connect(status.prog.on_load_progress)
         self.tabbed_browser.cur_load_started.connect(
             status.prog.on_load_started)
 
@@ -584,8 +596,7 @@ class MainWindow(QWidget):
         quit_texts = []
         # Ask if multiple-tabs are open
         if 'multiple-tabs' in config.val.confirm_quit and tab_count > 1:
-            quit_texts.append("{} {} open.".format(
-                tab_count, "tab is" if tab_count == 1 else "tabs are"))
+            quit_texts.append("{} tabs are open.".format(tab_count))
         # Ask if multiple downloads running
         if 'downloads' in config.val.confirm_quit and download_count > 0:
             quit_texts.append("{} {} running.".format(
